function [phi_fit]=photochemAB(data_filename,I0_filename,cA_0,phi_ini);
% Requieres Octave ver. 4.2.1 and the Optim-1.5.2 Octave-Forge package
% Type photochemAB('data.txt','I_0.txt',2.196e-5,0.2) in the Command Window.
% The function automatically generates E.txt file that contains the deconvoluted sprectra.

global A_exp;
global lambda;
global t;
global c_0;
global E;
global I_0;

data=load(data_filename);

% Spectral density of LED
I_0=load(I0_filename); %einstein.s^-1.dm^-3.nm^-1

% Initial reactant and product concentrations
c_0=[cA_0,0]; %M

% 1 x N_lambda array containing wavelegths in which the absorbance was
% measured
lambda=data(1,2:end); %nm

% N_M x 1 array containing times at which each measurement was performed
t=data(2:end,1); %s

% N_M x N_lambda array containing the absorbances measured
A_exp=data(2:end,2:end);

% Initial guess for quantum yield
phi=phi_ini; %einstein.s^-1

% N_S x N_lambda initial guess for the spectra of the colored species
E=[A_exp(1,:)/c_0(1);zeros(1,length(lambda))];

% Load required Octave package (contains the nlinfit function)
pkg load optim;

% Rearranges A_exp into a one-dimensional array
y=[];
for spec_num=1:1:size(A_exp,1);
    y=[y,A_exp(spec_num,:)];
end;

X=1;

% Sets tolerances and display options for the fitting procedure
options=optimset('Display','iter','TolFun',1e-8,'TolX',1e-8);

% Runs the non-linear fitting routine
phi_fit = nlinfit(X,y,@profile,phi,options);

%Fitted quantum yield:
phi_fit';
aux=[lambda;E]';
save 'E.txt' aux -ascii -tabs

end

function y=profile(phi,X)

global A_exp;
global t;
global c_0;
global C;
global A;
global E;
global I_0;

tolerance=10; 
    while tolerance>1e-4 % Thershold for convergence on E
    % Calculation of C based on the chemical model contained in funtion profileAB
    [t,C]=profileAB(t,phi,E,I_0,c_0);

    % Calculates E
    E_aux=pinv(C)*A_exp;

    % Evaluates change in E      
    tolerance=norm(E-E_aux);
    E=E_aux;
    end;

        A=C*E;
        
        y=[];
        for spec_num=1:size(A,1);
            y=[y,A(spec_num,:)];
        end;
end

function [t,C]=profileAB(t,phi,E,I_0,c_0)

global phi_aux;
global I0_exp;
global E_aux;

phi_aux=abs(phi);
I0_exp=I_0;
E_aux=E;

options = odeset('RelTol',1e-4,'AbsTol',[1e-7;1e-7]);
[t,C] = ode45(@dconc,t,c_0,options);

clear phi_aux;      
clear I0_exp;      
clear E_aux;     

end

function f=dconc(t,c);

global lambda
global phi_aux;
global I0_exp;
global E_aux;

Ea=E_aux(1,:);
Eb=E_aux(2,:);

f=zeros(2,1);

% Total light absorbed
Ia_tot=I0_exp.*(1-10.^(-Ea*c(1)-Eb*c(2)));

% Fraction of light absorbed by each species;
factorA=Ea*c(1)./(Ea*c(1)+Eb*c(2));
factorB=Eb*c(2)./(Ea*c(1)+Eb*c(2));

% Differential equations (dc/dt) 
f(1)=(lambda(2)-lambda(1))*sum(-factorA.*phi_aux(1).*Ia_tot);
f(2)=(lambda(2)-lambda(1))*sum(factorA.*phi_aux(1).*Ia_tot);
end